# -*- coding: utf-8 -*-

#   Didacto, un logiciel d'aide 脿 l'organisation d'un corpus didactique
#   Copyright (C) 2020  Marco de Freitas
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.
#   If not, see <https://www.gnu.org/licenses/>.
#
#    contact:marco@sillage.ch

from pathlib import Path
import os
import sys
import io
import configparser
import json
import subprocess

from PyPDF2 import PdfFileReader, PdfFileWriter

from view import *
from model import *

ROOT_FOLDER = "didacto"
DATA_FOLDER = "data"
DATA_FILE = "data.did"
CONFIG_FOLDER = "config"
CONFIG_FILE = "preferences.ini"





def create_default_file_struct(home_dir):
    """creat prefs and user data file structure"""
    print("Creating fs struct because it doesn't exist yet")
    root_folder = os.path.join(home_dir, ROOT_FOLDER)
    data_folder = os.path.join(root_folder, DATA_FOLDER)
    config_folder = os.path.join(root_folder, CONFIG_FOLDER)
    data_file_path = os.path.join(data_folder, DATA_FILE)
    config_file_path = os.path.join(config_folder, CONFIG_FILE)

    try:
        os.mkdir(root_folder)
    except FileExistsError:
        pass

    try:
        os.mkdir(data_folder)
    except FileExistsError:
        pass
    try:
        os.mkdir(config_folder)
    except FileExistsError:
        pass
    Path(data_file_path).touch()
    Path(config_file_path).touch()


def init_config_file(prefs_path):
    """initialise prefs files with structure and default values"""
    config = configparser.ConfigParser()
    config['DEFAULT'] = {'path': '', 'separator': '', 'notation_format': '.ly'}
    config['USER'] = {}
    with io.open(prefs_path, 'w') as configfile:
        config.write(configfile)


def init_user_data_file(data_path):
    """initialise user data file"""
    data = {}
    y = json.dumps(data)
    with io.open(data_path, 'w') as file:
        file.write(y)


def open_user_data(path):
    with io.open(path) as file:
        try:
            a = file.read()
            x = json.loads(a)
        except json.decoder.JSONDecodeError:
            # file is corrupted, initialize file
            init_user_data_file(path)
            a = file.read()
            x = json.loads(a)
        return x


class Controller:
    def __init__(self):
        """Initialise the MVC structure."""
        # Initialize home dir, user data and config files paths
        self.home_dir = str(Path.home())
        self.user_data_path = os.path.join(self.home_dir, ROOT_FOLDER, DATA_FOLDER, DATA_FILE)
        self.config_file_path = os.path.join(self.home_dir, ROOT_FOLDER, CONFIG_FOLDER, CONFIG_FILE)

        # check if default file tree exists, if not, creat it
        if not Path(self.config_file_path).exists() or not Path(self.user_data_path).exists():
            create_default_file_struct(self.home_dir)

        # load prefs and user data
        prefs = self.open_saved_prefs()
        user_data = open_user_data(self.user_data_path)

        # Initialize MVC
        self.root = tkinter.Tk()
        self.model = Model(prefs, user_data)
        self.view = View(self.root, self.model, self)

    def run(self):
        """This function starts the windowing mechanism."""
        self.root.title("Didacto")
        self.root.deiconify()
        self.root.mainloop()

    def quit(self):
        """This fuction quits the programm."""
        self.root.destroy()

    def open_saved_prefs(self):
        """Opens preferences.ini file, return preferences keys and values."""
        config = configparser.ConfigParser()
        config.read(self.config_file_path)
        preferences = {}
        try:
            uservalues = config['USER']
        except KeyError:
            # file is corrupted, rewrite correct data structure
            init_config_file(self.config_file_path)
            blanck_user_prefs = {'path': '', 'separator': '', 'notation_format': ''}
            self.save_new_user_prefs(blanck_user_prefs)
            config.read(self.config_file_path)
            uservalues = config['USER']

        for i in config['USER']:
            preferences[i] = uservalues.get(i)
        return preferences

    def save_new_user_prefs(self, prefs):
        """save new user preferences"""
        config = configparser.ConfigParser()
        config['DEFAULT'] = {'path': '', 'separator': '', 'notation_format': '.ly'}
        config['USER'] = prefs
        with io.open(self.config_file_path, 'w') as configfile:
            config.write(configfile)

    def write_user_data(self):
        y = self.model.user_data.retrieve()
        dump = json.dumps(y)
        with io.open(self.user_data_path, 'w') as file:
            file.write(dump)

    # Code copié de:
    # http://sametmax.com/ouvrir-un-fichier-avec-le-bon-programme-en-python/
    def run_file(self, path):
        """system call open file."""
        if not os.path.exists(path):
            raise OSError
        # ('No such file: %s' % path) TODO replace error message by tkinter.messagebox.showwarning
        #
        # if hasattr(os, 'access') and not os.access(path, os.R_OK):
        #     raise IOError('Cannot access file: %s' % path)

        # os dependant open function

        if hasattr(os, 'startfile'):  # Windows
            proc = os.startfile(path)
        elif sys.platform.startswith('linux'):  # Linux:
            proc = subprocess.Popen(['xdg-open', path],
                                    # on capture stdin et out pour
                                    # rendre le tout non bloquant
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        elif sys.platform == 'darwin':  # Mac:
            proc = subprocess.Popen(['open', '--', path],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        else:
            raise NotImplementedError(
                "Your `%s` isn't a supported operating system`." % sys.platform)

        return proc

    @staticmethod
    def edit_pdf_keywords(path, keywords):
        with io.open(path, 'rb') as file_in:
            reader = PdfFileReader(file_in)
            writer = PdfFileWriter()

            metadata_in = reader.getDocumentInfo()
            metadata_out = {}
            for key in metadata_in:
                metadata_out[key] = metadata_in[key]
            if '/AAPL:Keywords' in metadata_out:
                metadata_out.pop('/AAPL:Keywords')  # this fixes some bug with sibelius6 - MacOs pdf
            # print(metadata_out)
            # print(type(metadata_out))

            writer.appendPagesFromReader(reader)
            writer.addMetadata(metadata_out)
            writer.addMetadata({
                '/Keywords': str(keywords)
            }) # for some reason, keywords string must be converted to string

            with io.open(path, 'ab') as file_out:  # ab is append binary; if you do wb, the file will append blank pages
                writer.write(file_out)
